@@ -4,7 +4,7 @@ Client for interacting with the Neuromorphic Computing Platform of the Human Bra
 Authors: Andrew P. Davison, Domenico Guarino, UNIC, CNRS
 
 
-Copyright 2016-2018 Andrew P. Davison and Domenico Guarino, Centre National de la Recherche Scientifique
+Copyright 2016-2020 Andrew P. Davison and Domenico Guarino, Centre National de la Recherche Scientifique
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -38,10 +38,14 @@ import mimetypes
 import requests
 from requests.auth import AuthBase
 try:
-    from jupyter_collab_storage import oauth_token_handler
+    from jupyter_collab_storage import oauth_token_handler  # v1
     have_collab_token_handler = True
 except ImportError:
-    have_collab_token_handler = False
+    try:
+        from clb_nb_utils import oauth as oauth_token_handler  # v2
+        have_collab_token_handler = True
+    except ImportError:
+        have_collab_token_handler = False
 try:
     from hbp_service_client.storage_service.client import Client as StorageClient
     from hbp_service_client.storage_service.exceptions import StorageException
@@ -52,8 +56,10 @@ except ImportError:
 mimetypes.init()
 logger = logging.getLogger("NMPI")
 
-IDENTITY_SERVICE = "https://services.humanbrainproject.eu/idm/v1/api"
-COLLAB_SERVICE = "https://services.humanbrainproject.eu/collab/v0"
+IDENTITY_SERVICE_V1 = "https://services.humanbrainproject.eu/idm/v1/api"
+IDENTITY_SERVICE_V2 = "https://iam.ebrains.eu/auth/realms/hbp/protocol/openid-connect"
+COLLAB_SERVICE_V1 = "https://services.humanbrainproject.eu/collab/v0"
+COLLAB_SERVICE_V2 = "https://wiki.ebrains.eu/rest/v1/"
 
 TOKENFILE = os.path.expanduser("~/.hbptoken")
 
@@ -151,8 +157,10 @@ class Client(object):
             else:
                 raise
 
-        with open(TOKENFILE, "w") as fp:
-            json.dump({username: {"access_token": self.token}}, fp)
+        if not have_collab_token_handler:
+            # no need to cache the token if running in the Collaboratory
+            with open(TOKENFILE, "w") as fp:
+                json.dump({username: {"access_token": self.token}}, fp)
 
         # get schema
         req = requests.get(job_service, cert=self.cert, verify=self.verify, auth=self.auth)
@@ -166,6 +174,7 @@ class Client(object):
     def _hbp_auth(self, username, password):
         """
         """
+        # todo: add support for v2
         redirect_uri = self.job_server + '/complete/hbp/'
 
         self.session = requests.Session()
@@ -234,16 +243,23 @@ class Client(object):
             raise Exception("Something went wrong. Status code {} from NMPI, expected 302".format(rNMPI1.status_code))
 
     def _get_user_info(self):
-        req = requests.get(IDENTITY_SERVICE + "/user/me",
+        req = requests.get(IDENTITY_SERVICE_V1 + "/user/me",
                            auth=self.auth)
         if req.ok:
             self.user_info = req.json()
-            if self.username:
-                assert self.user_info['username'] == self.username
-            else:
-                self.username = self.user_info['username']
         else:
-            self._handle_error(req)
+            req2 = requests.get(IDENTITY_SERVICE_V2 + "/userinfo",
+                                auth=self.auth)
+            if req2.ok:
+                self.user_info = req2.json()
+                self.user_info["id"] = self.user_info.get("preferred_username", self.user_info["sub"])
+                self.user_info["username"] = self.user_info.get("preferred_username", "unknown")
+            else:
+                self._handle_error(req)  # todo: need to somehow combine req and req2 here
+        if self.username:
+            assert self.user_info['username'] == self.username
+        else:
+            self.username = self.user_info['username']
 
     def _handle_error(self, request):
         """
@@ -289,7 +305,6 @@ class Client(object):
                             auth=self.auth,
                             cert=self.cert, verify=self.verify,
                             headers={"content-type": "application/json"})
-        #import pdb; pdb.set_trace()
         if not req.ok:
             self._handle_error(req)
         if 'Location' in req.headers:
@@ -597,7 +612,7 @@ class Client(object):
         *Returns*:
             the entity id of the remote folder
         """
-
+        # todo: add support for Drive
         if not self.storage_client:
             if have_collab_client:
                 self.storage_client = StorageClient.new(self.token)
@@ -686,18 +701,22 @@ class Client(object):
         """
         Return a list of collabs of which the user is a member.
         """
-        collabs = []
-        next = COLLAB_SERVICE + '/mycollabs'
-        while next:
-            req = requests.get(next, auth=self.auth)
-            if req.ok:
-                data = req.json()
-                next = data["next"]
-                collabs.extend(data["results"])
-            else:
-                self._handle_error(req)
-        return dict((c["title"], c)
-                    for c in collabs if not c["deleted"])
+        if "roles" in self.user_info:
+            return self.user_info["roles"]["team"]
+            # todo: retrieve more information, like Collab names
+        else:
+            collabs = []
+            next = COLLAB_SERVICE_V1 + '/mycollabs'
+            while next:
+                req = requests.get(next, auth=self.auth)
+                if req.ok:
+                    data = req.json()
+                    next = data["next"]
+                    collabs.extend(data["results"])
+                else:
+                    self._handle_error(req)
+            return dict((c["title"], c)
+                        for c in collabs if not c["deleted"])
 
     def create_resource_request(self, title, collab_id, abstract, description=None, submit=False):
         """
@@ -705,19 +724,25 @@ class Client(object):
         """
 
         context = str(uuid.uuid4())
+        try:
+            int(collab_id)
+            version = 1
+        except ValueError:
+            version = 2
 
-        # create a new nav item in the Collab
-        nav_resource = COLLAB_SERVICE + "/collab/{}/nav/".format(collab_id)
-        nav_root = self._query(nav_resource + "root")['id']
-        nav_item = {
-            "app_id": "206",
-            "context": context,
-            "name": "Resource request",
-            "order_index": "-1",
-            "parent": nav_root,
-            "type": "IT"
-        }
-        result = self._post(nav_resource, nav_item)
+        if version == 1:
+            # create a new nav item in the Collab
+            nav_resource = COLLAB_SERVICE_V1 + "/collab/{}/nav/".format(collab_id)
+            nav_root = self._query(nav_resource + "root")['id']
+            nav_item = {
+                "app_id": "206",
+                "context": context,
+                "name": "Resource request",
+                "order_index": "-1",
+                "parent": nav_root,
+                "type": "IT"
+            }
+            result = self._post(nav_resource, nav_item)
 
         # create the resource request
         new_project = {
